@@ -29,7 +29,6 @@
 #
 # Standard library
 # ----------------
-from pathlib import PurePosixPath
 import socket
 import threading
 from typing import Union
@@ -70,6 +69,8 @@ HTTP_PORT = 5000
 # The port used for the Thrift connection between text editor/IDE extensions/plugins and the CodeChat Server. All editor/IDE plugins must use this port to access CodeChat services.
 THRIFT_PORT = 9090
 
+UNKNOWN_CLIENT = "Unknown client id {}."
+
 
 # Service provider
 # ================
@@ -80,11 +81,13 @@ class CodeChatHandler:
 
     # _`get_client`: Return the HTML for a web client.
     def get_client(self, codeChat_client_location: int) -> RenderClientReturn:
-        print(
-            "get_client({})".format(
-                CodeChatClientLocation._VALUES_TO_NAMES[codeChat_client_location]
-            )
-        )
+        try:
+            location_name = CodeChatClientLocation._VALUES_TO_NAMES[
+                codeChat_client_location
+            ]
+        except KeyError:
+            location_name = "invalid location"
+        print("get_client({})".format(location_name))
         id = self.render_manager.threadsafe_create_client()
         # Get the next ID.
         if id is None:
@@ -120,7 +123,7 @@ class CodeChatHandler:
             ret_str = ""
         else:
             ret = RenderClientReturn(
-                "", -1, "Unknown command {}".format(codeChat_client_location)
+                "", -1, "Invalid location {}".format(codeChat_client_location)
             )
             print("  => {}".format(ret))
             return ret
@@ -141,7 +144,7 @@ class CodeChatHandler:
             print(" => (empty string)")
             return ""
         else:
-            ret = "Unknown client id {}".format(id)
+            ret = UNKNOWN_CLIENT.format(id)
             print(" => {}".format(ret))
             return ret
 
@@ -155,7 +158,7 @@ class CodeChatHandler:
         print("stop_client(id={})".format(id))
         ok = self.render_manager.threadsafe_shutdown_client(id)
         if not ok:
-            ret = "unknown client {}.".format(id)
+            ret = UNKNOWN_CLIENT.format(id)
             print("  => {}".format(ret))
             return ret
         # Indicate success.
@@ -192,7 +195,7 @@ def editor_plugin_server() -> None:
     #
     # This server spawns a thread per connection.
     ## server = TServer.TThreadedServer(processor, transport, tfactory, pfactory)  # noqa: E266
-    # An simpler server:
+    # A simpler server:
     ## server = TServer.TSimpleServer(processor, transport, tfactory, pfactory)  # noqa: E266
 
     server.serve()
@@ -223,32 +226,22 @@ def client_html() -> str:
 def client_data(id: int, url_path: str) -> Union[str, Response]:
     # See if we rendered this file.
     html = handler.render_manager.threadsafe_get_render_results(id, url_path)
+
     # If we have rendered HTML, return it.
     if type(html) == str:
         assert isinstance(html, str)
         response = make_response(html)
-        # Don't allow the browser to cache files. See the `Flask docs <https://flask.palletsprojects.com/en/1.1.x/api/#flask.Request.cache_control>`_ and `MDN docs <https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Cache-Control#Directives>`_. TODO: allow caching of static files (but how do we know which files are static?)
-        response.cache_control.no_store = True
-        return response
-
-    # If this render was a project, then ``html`` is None. In this case, the rendered HTML is already on disk at ``url_path``; however, don't allow Flask to cache this, since it changes with each edit.
-    if html is None:
-        send_file_kwargs = dict(cache_timeout=0)
     else:
-        send_file_kwargs = {}
+        # The file is on disk. Send it or a 404 if nothing was found.
+        try:
+            # Don't allow Flask to cache files on disk, since they may change with each edit.
+            response = make_response(send_file(url_path, cache_timeout=0))  # type: ignore
+        except (FileNotFoundError, PermissionError):
+            abort(404)
 
-    # Send a static file or a 404 if nothing was found.
-    try:
-        # TODO SECURITY: if a web app, need to limit the base directory to wherever projects are placed on disk.
-        response = make_response(send_file(url_path, **send_file_kwargs))  # type: ignore
-        # By default, allow caching on everything but HTML files.
-        dont_cache = PurePosixPath(url_path).suffix in (".htm", ".html")
-        if dont_cache:
-            # See same code above.
-            response.cache_control.no_store = True
-        return response
-    except (FileNotFoundError, PermissionError):
-        abort(404)
+    # Don't allow the browser to cache files. See the `Flask docs <https://flask.palletsprojects.com/en/1.1.x/api/#flask.Request.cache_control>`_ and `MDN docs <https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Cache-Control#Directives>`_.
+    response.cache_control.no_store = True
+    return response
 
 
 # Main code
@@ -307,7 +300,11 @@ def run_servers() -> int:
     return 0
 
 
-# Copied verbatim from Stack Overflow.
+# Inspired by Stack Overflow. The original post used ``connect_ex``, which is very slow (~2 seconds).
 def is_port_in_use(port):
     with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as s:
-        return s.connect_ex(("localhost", port)) == 0
+        try:
+            s.bind(("localhost", port))
+        except OSError:
+            return True
+        return False
